@@ -53,6 +53,10 @@
 #include "ssl.h"
 #endif
 
+#ifdef USE_MSPROXY
+#include "msproxy.h"
+#endif
+
 GSList *popup_list = 0;
 GSList *button_list = 0;
 GSList *dlgbutton_list = 0;
@@ -72,6 +76,12 @@ int xchat_is_quitting = FALSE;
 int arg_dont_autoconnect = FALSE;
 int arg_skip_plugins = FALSE;
 char *arg_url = NULL;
+gint arg_existing = FALSE;
+
+#ifdef USE_DBUS
+#include "dbus/dbus-client.h"
+#include "dbus/dbus-plugin.h"
+#endif /* USE_DBUS */
 
 struct session *current_tab;
 struct session *current_sess = 0;
@@ -245,6 +255,9 @@ static int
 xchat_misc_checks (void)		/* this gets called every 1/2 second */
 {
 	static int count = 0;
+#ifdef USE_MSPROXY
+	static int count2 = 0;
+#endif
 
 	count++;
 
@@ -260,6 +273,15 @@ xchat_misc_checks (void)		/* this gets called every 1/2 second */
 		count = 0;
 	}
 
+#ifdef USE_MSPROXY	
+	count2++;
+	if (count2 >= 720)			/* 720 every 6 minutes */
+	{
+		msproxy_keepalive ();
+		count2 = 0;
+	}
+#endif
+
 	return 1;
 }
 
@@ -279,6 +301,11 @@ irc_init (session *sess)
 	if (!arg_skip_plugins)
 		plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
 #endif
+
+#ifdef USE_DBUS
+	plugin_add (sess, NULL, NULL, dbus_plugin_init, NULL, NULL, FALSE);
+#endif
+
 	plugin_add (sess, NULL, NULL, timer_plugin_init, NULL, NULL, FALSE);
 
 	if (prefs.notify_timeout)
@@ -417,7 +444,8 @@ send_quit_or_part (session * killsess)
 			}
 		} else
 		{
-			if (killsess->type == SESS_CHANNEL && killsess->channel[0])
+			if (killsess->type == SESS_CHANNEL && killsess->channel[0] &&
+				 !killserv->sent_quit)
 			{
 				server_sendpart (killserv, killsess->channel, 0);
 			}
@@ -486,8 +514,10 @@ session_free (session *killsess)
 
 	free (killsess);
 
+#if 0
 	if (!sess_list && !in_xchat_exit)
 		xchat_exit ();						/* sess_list is empty, quit! */
+#endif
 
 	list = sess_list;
 	while (list)
@@ -883,7 +913,7 @@ xchat_init (void)
 		"NAME %s\n"		"CMD op %%a\n\n"
 		"NAME %s\n"		"CMD deop %%a\n\n"
 		"NAME %s\n"		"CMD ban %%s\n\n"
-		"NAME %s\n"		"CMD getstr %s \"kick %%s\" \"%s\"\n\n"
+		"NAME %s\n"		"CMD getstr \"%s\" \"kick %%s\" \"%s\"\n\n"
 		"NAME %s\n"		"CMD send %%s\n\n"
 		"NAME %s\n"		"CMD query %%s\n\n",
 				_("Op"),
@@ -1007,6 +1037,10 @@ main (int argc, char *argv[])
 	ret = fe_args (argc, argv);
 	if (ret != -1)
 		return ret;
+	
+#ifdef USE_DBUS
+	xchat_remote ();
+#endif
 
 	load_config ();
 
