@@ -64,9 +64,9 @@ static void     conversation_panel_enter_word         (GtkWidget              *x
 static void     conversation_panel_leave_word         (GtkWidget              *xtext,
                                                        char                   *word,
                                                        ConversationPanel      *panel);
-static gboolean	conversation_panel_lost_focus	      (GtkWidget	      *widget,
-						       GdkEventFocus	      *event,
-						       ConversationPanel      *panel);
+static gboolean conversation_panel_lost_focus         (GtkWidget              *widget,
+                                                       GdkEventFocus          *event,
+                                                       ConversationPanel      *panel);
 #endif
 static void     conversation_panel_set_font           (ConversationPanel      *panel);
 static void     conversation_panel_font_changed       (GConfClient            *client,
@@ -87,7 +87,7 @@ static void     redundant_nickstamp_changed           (GConfClient            *c
                                                        GConfEntry             *entry,
                                                        ConversationPanel      *panel);
 static void     conversation_panel_print_line         (ConversationPanel      *panel,
-		                                       xtext_buffer           *buffer,
+                                                       xtext_buffer           *buffer,
                                                        guchar                  *text,
                                                        int                     len,
                                                        gboolean                indent);
@@ -109,7 +109,7 @@ static void     drop_paste_filename                   (GtkAction              *a
                                                        ConversationPanel      *panel);
 static void     drop_cancel                           (GtkAction              *action,
                                                        ConversationPanel      *panel);
-static void		style_set_callback                    (GtkWidget              *widget,
+static void     style_set_callback                    (GtkWidget              *widget,
                                                        GtkStyle               *previous_style,
                                                        void                   *data);
 static void     drag_data_received                    (GtkWidget              *widget,
@@ -130,7 +130,7 @@ static void     on_default_copy_activate              (GtkAction              *a
 static gboolean uri_is_text                           (gchar                  *uri);
 static gboolean check_file_size                       (gchar                  *uri);
 #ifdef HAVE_LIBSEXY
-GtkWidget* get_user_vbox_infos	    	    	      (struct User            *user);
+GtkWidget* get_user_vbox_infos                        (struct User            *user);
 #endif
 static void redraw_transparency                       (ConversationPanel      *panel);
 #ifdef USE_XLIB
@@ -160,15 +160,15 @@ struct _ConversationPanelPriv
 	GSList         *dropped_files;
 
 #ifdef HAVE_LIBSEXY
-	GtkWidget	*current_tooltip;
-	guint		tooltip_timeout;
+	GtkWidget      *current_tooltip;
+	guint           tooltip_timeout;
 #endif
-	gboolean	redundant_nickstamps;
+	gboolean        redundant_nickstamps;
 };
 
 static GtkActionEntry url_actions[] = {
 	{ "TextURLOpen", GTK_STOCK_OPEN, N_("_Open Link in Browser"), NULL, NULL, G_CALLBACK (open_url) },
-        { "TextURLCopy", GTK_STOCK_COPY, N_("_Copy Link Location"),   NULL, NULL, G_CALLBACK (copy_text) },
+	{ "TextURLCopy", GTK_STOCK_COPY, N_("_Copy Link Location"),   NULL, NULL, G_CALLBACK (copy_text) },
 };
 
 static GtkActionEntry email_actions[] = {
@@ -256,9 +256,9 @@ conversation_panel_init (ConversationPanel *panel)
 
 	action_group = gtk_action_group_new ("TextPopups");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (action_group, url_actions,   G_N_ELEMENTS (url_actions),   panel);
-	gtk_action_group_add_actions (action_group, email_actions, G_N_ELEMENTS (email_actions), panel);
-	gtk_action_group_add_actions (action_group, dnd_actions,   G_N_ELEMENTS (dnd_actions),   panel);
+	gtk_action_group_add_actions (action_group, url_actions,     G_N_ELEMENTS (url_actions),   panel);
+	gtk_action_group_add_actions (action_group, email_actions,   G_N_ELEMENTS (email_actions), panel);
+	gtk_action_group_add_actions (action_group, dnd_actions,     G_N_ELEMENTS (dnd_actions),   panel);
 	gtk_action_group_add_actions (action_group, default_actions, G_N_ELEMENTS (default_actions),   panel);
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
@@ -272,7 +272,7 @@ conversation_panel_init (ConversationPanel *panel)
 		gconf_client_get_bool (client, "/apps/xchat/main_window/redundant_nickstamps", NULL);
 	g_object_unref (client);
 
-	g_signal_connect (G_OBJECT (panel), "style_set", G_CALLBACK (style_set_callback), panel);
+	g_signal_connect (G_OBJECT (panel), "style-set", G_CALLBACK (style_set_callback), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "drag_data_received", G_CALLBACK (drag_data_received), panel);
 	gtk_drag_dest_set (panel->priv->xtext, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 	                   target_table, G_N_ELEMENTS (target_table), GDK_ACTION_COPY | GDK_ACTION_ASK);
@@ -281,6 +281,8 @@ conversation_panel_init (ConversationPanel *panel)
 	gdk_window_set_events (gdk_get_default_root_window (), GDK_PROPERTY_CHANGE_MASK);
 	gdk_window_add_filter (gdk_get_default_root_window (), (GdkFilterFunc)root_event_cb, panel);
 #endif
+
+	conversation_panel_set_show_marker (panel, prefs.show_marker);
 }
 
 static void
@@ -294,8 +296,9 @@ conversation_panel_finalize (GObject *object)
 	g_hash_table_destroy (panel->priv->timestamp_notifies);
 	g_free (panel->priv);
 
-	if (G_OBJECT_CLASS (parent_class)->finalize)
+	if (G_OBJECT_CLASS (parent_class)->finalize) {
 		G_OBJECT_CLASS (parent_class)->finalize (object);
+	}
 }
 
 static void
@@ -304,8 +307,9 @@ conversation_panel_realize (GtkWidget *widget)
 	ConversationPanel *panel;
 	GConfClient       *client;
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize)
+	if (GTK_WIDGET_CLASS (parent_class)->realize) {
 		GTK_WIDGET_CLASS (parent_class)->realize (widget);
+	}
 
 	panel  = CONVERSATION_PANEL (widget);
 	client = gconf_client_get_default ();
@@ -323,11 +327,12 @@ conversation_panel_realize (GtkWidget *widget)
 	conversation_panel_set_font       (panel);
 	conversation_panel_set_background (panel);
 
-	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click", G_CALLBACK (conversation_panel_clicked_word), panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click",      G_CALLBACK (conversation_panel_clicked_word), panel);
 #ifdef HAVE_LIBSEXY
-	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter", G_CALLBACK (conversation_panel_enter_word), panel);
-	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_leave", G_CALLBACK (conversation_panel_leave_word), panel);
-	g_signal_connect (G_OBJECT (gui.main_window), "focus-out-event", G_CALLBACK (conversation_panel_lost_focus), panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter",      G_CALLBACK (conversation_panel_enter_word),   panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_leave",      G_CALLBACK (conversation_panel_leave_word),   panel);
+	g_signal_connect (G_OBJECT (gui.main_window),    "focus-out-event", G_CALLBACK (conversation_panel_lost_focus),   panel);
+	g_signal_connect (G_OBJECT (gui.main_window),    "leave-notify-event", G_CALLBACK (conversation_panel_lost_focus),   panel);
 #endif
 	gconf_client_notify_add (client, "/apps/xchat/main_window/use_sys_fonts",
 	                         (GConfClientNotifyFunc) conversation_panel_font_changed,       panel, NULL, NULL);
@@ -355,13 +360,15 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
 	url = url_check_word (word, len);
 	if (url == 0 && current_sess) {
 		if (current_sess->type == SESS_DIALOG) {
-			if (strcmp (word, current_sess->channel) == 0)
+			if (strcmp (word, current_sess->channel) == 0) {
 				return WORD_NICK;
+			}
 			return WORD_DIALOG;
 		} else if (((word[0]=='@' || word[0]=='+') && userlist_find (current_sess, word+1)) ||
 			   userlist_find (current_sess, word)) {
-			if (strcmp (word, current_sess->server->nick) != 0)
+			if (strcmp (word, current_sess->server->nick) != 0) {
 				return WORD_NICK;
+			}
 		}
 	}
 	return url;
@@ -370,8 +377,9 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
 static void
 conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *event, ConversationPanel *panel)
 {
-	if (word == NULL)
+	if (word == NULL) {
 		return;
+	}
 
 	if (event->button == 1) {
 		switch (conversation_panel_check_word (xtext, word, strlen (word))) {
@@ -390,13 +398,14 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct session *sess;
 
 				if ((gui.current_session->type == SESS_DIALOG) &&
-				    (strcmp (gui.current_session->channel, word) == 0))
+				    (strcmp (gui.current_session->channel, word) == 0)) {
 					break;
+				}
 
 				sess = find_dialog (gui.current_session->server, word);
-				if (sess)
+				if (sess) {
 					navigation_tree_select_session (gui.server_tree, sess);
-				else {
+				} else {
 					command = g_strdup_printf ("QUERY %s", word);
 					handle_command (gui.current_session, command, 1);
 					g_free (command);
@@ -409,8 +418,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct session *sess;
 
 				sess = find_channel (gui.current_session->server, word);
-				if (sess)
+				if (sess) {
 					navigation_tree_select_session (gui.server_tree, sess);
+				}
 				/* Having the channel opened doesn't mean that we're still on it
 				 * (if channel was left) */
 				command = g_strdup_printf ("JOIN %s", word);
@@ -445,8 +455,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 			{
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextURLPopup");
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
@@ -456,8 +467,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct User *user;
 				GtkWidget   *menu;
 
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 
 				if (gui.current_session->type == SESS_CHANNEL) {
@@ -483,8 +495,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 			{
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextEmailPopup");
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
@@ -504,11 +517,11 @@ show_user_tooltip (gchar *nick)
 	GdkScreen *screen;
 	GtkWidget *vbox;
 	GdkDisplay *display;
-	
+
 	panel = CONVERSATION_PANEL(gui.conversation_panel);
 
 	user = userlist_find (gui.current_session, nick);
-	
+
 	if (user) {
 		tooltip = sexy_tooltip_new ();
 
@@ -517,7 +530,7 @@ show_user_tooltip (gchar *nick)
 		rect.y += 10;
 		rect.width = 1;
 		rect.height = 1;
-		
+
 		vbox = get_user_vbox_infos (user);
 		gtk_container_add (GTK_CONTAINER (tooltip), vbox);
 		sexy_tooltip_position_to_rect (SEXY_TOOLTIP (tooltip), &rect, screen);
@@ -533,12 +546,11 @@ static void
 conversation_panel_enter_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
 {
 	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
-		case WORD_NICK:
-			{
-				panel->priv->tooltip_timeout =  
-					g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
-				break;
-			}
+	case WORD_NICK:
+		{
+			panel->priv->tooltip_timeout = g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
+			break;
+		}
 	}
 }
 
@@ -560,11 +572,11 @@ static void
 conversation_panel_leave_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
 {
 	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
-		case WORD_NICK:
-			{
-				conversation_panel_remove_tooltip (panel);
-				break;
-			}
+	case WORD_NICK:
+		{
+			conversation_panel_remove_tooltip (panel);
+			break;
+		}
 	}
 }
 
@@ -583,13 +595,15 @@ conversation_panel_set_font (ConversationPanel *panel)
 	gchar       *font = NULL;
 
 	client = gconf_client_get_default ();
-	if (!gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL))
+	if (!gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL)) {
 		font = gconf_client_get_string (client, "/apps/xchat/main_window/font", NULL);
+	}
 
 	/* Either use_sys_fonts==TRUE, or there is no current font preference.
 	 * In both cases we try the GNOME monospace font. */
-	if (font == NULL)
+	if (font == NULL) {
 		font = gconf_client_get_string (client, "/desktop/gnome/interface/monospace_font_name", NULL);
+	}
 
 	g_object_unref (client);
 
@@ -694,13 +708,15 @@ copy_text (GtkAction *action, ConversationPanel *panel)
 static void
 send_email (GtkAction *action, ConversationPanel *panel)
 {
+	/* FIXME */
 }
 
 static void
 drop_send_files  (GtkAction *action, ConversationPanel *panel)
 {
-	if (panel->priv->current->type != SESS_DIALOG)
+	if (panel->priv->current->type != SESS_DIALOG) {
 		return;
+	}
 	drop_send (panel);
 }
 
@@ -720,8 +736,9 @@ drop_paste_filename  (GtkAction *action, ConversationPanel *panel)
 
 	for (l = panel->priv->dropped_files; l != NULL; l = g_slist_next (l)) {
 		path = g_filename_from_uri (l->data, NULL, NULL);
-		if (path == NULL)
+		if (path == NULL) {
 			path = g_strdup (l->data);
+		}
 
 		if (txt == NULL) {
 			txt = g_strdup (path);
@@ -735,8 +752,9 @@ drop_paste_filename  (GtkAction *action, ConversationPanel *panel)
 		g_free (path);
 	}
 
-	if (panel->priv->current != NULL)
+	if (panel->priv->current != NULL) {
 		handle_multiline (panel->priv->current, txt, TRUE, FALSE);
+	}
 
 	g_free (txt);
 	free_dropped_files (panel);
@@ -762,8 +780,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		guchar *txt;
 
 		txt = gtk_selection_data_get_text (selection_data);
-		if (gui.current_session != NULL)
-		handle_multiline (gui.current_session, txt, TRUE, FALSE);
+		if (gui.current_session != NULL) {
+			handle_multiline (gui.current_session, txt, TRUE, FALSE);
+		}
 
 		g_free (txt);
 		break;
@@ -775,8 +794,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		gint nb_uri;
 
 		if ((panel->priv->current->type != SESS_CHANNEL) &&
-		    (panel->priv->current->type != SESS_DIALOG))
+		    (panel->priv->current->type != SESS_DIALOG)) {
 			return;
+		}
 
 		if (selection_data->format != 8 || selection_data->length == 0) {
 			g_printerr (_("URI list dropped on xchat-gnome had wrong format (%d) or length (%d)\n"),
@@ -790,8 +810,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 
 		free_dropped_files (panel);
 
-		for (nb_uri = 0; uris[nb_uri] && strlen (uris[nb_uri]) > 0; nb_uri++)
+		for (nb_uri = 0; uris[nb_uri] && strlen (uris[nb_uri]) > 0; nb_uri++) {
 			panel->priv->dropped_files = g_slist_prepend (panel->priv->dropped_files, uris[nb_uri]);
+		}
 		g_free (uris); /* String in uris will be freed in free_dropped_files */
 		panel->priv->dropped_files = g_slist_reverse (panel->priv->dropped_files);
 
@@ -803,17 +824,19 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 			entry = gtk_ui_manager_get_widget (gui.manager, "/DropFilePopup/DropPasteFile");
 			if (nb_uri > 1 ||
 			    (uri_is_text (panel->priv->dropped_files->data) == FALSE) ||
-			    (check_file_size (panel->priv->dropped_files->data) == FALSE))
+			    (check_file_size (panel->priv->dropped_files->data) == FALSE)) {
 				gtk_widget_set_sensitive (entry, FALSE);
-			else
+			} else {
 				gtk_widget_set_sensitive (entry, TRUE);
+			}
 
 			/* Enable/Disable send files */
 			entry = gtk_ui_manager_get_widget (gui.manager, "/DropFilePopup/DropSendFiles");
-			if (panel->priv->current->type == SESS_CHANNEL)
+			if (panel->priv->current->type == SESS_CHANNEL) {
 				gtk_widget_set_sensitive (entry, FALSE);
-			else
+			} else {
 				gtk_widget_set_sensitive (entry, TRUE);
+			}
 
 			gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 2, gtk_get_current_event_time ());
 		} else {
@@ -822,16 +845,18 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 				/* Dropped in a channel */
 				if (nb_uri == 1 &&
 				    uri_is_text (panel->priv->dropped_files->data) &&
-				    check_file_size (panel->priv->dropped_files->data))
+				    check_file_size (panel->priv->dropped_files->data)) {
 					drop_paste (panel);
+				}
 			} else {
 				/* Dropped in a query */
 				if (nb_uri == 1 &&
 				    uri_is_text (panel->priv->dropped_files->data) &&
-				    check_file_size (panel->priv->dropped_files->data))
+				    check_file_size (panel->priv->dropped_files->data)) {
 					drop_paste (panel);
-				else
+				} else {
 					drop_send (panel);
+				}
 			}
 		}
 	}
@@ -853,7 +878,7 @@ drop_send (ConversationPanel *panel)
 {
 	g_return_if_fail (panel->priv->dropped_files != NULL);
 	g_slist_foreach (panel->priv->dropped_files, send_file, NULL);
-        free_dropped_files (panel);
+	free_dropped_files (panel);
 }
 
 static void
@@ -867,8 +892,9 @@ drop_paste (ConversationPanel *panel)
 	uri = panel->priv->dropped_files->data;
 	result = gnome_vfs_read_entire_file (uri, NULL, &contents);
 	if (result == GNOME_VFS_OK) {
-		if (panel->priv->current != NULL)
+		if (panel->priv->current != NULL) {
 			handle_multiline (panel->priv->current, (char *) contents, TRUE, FALSE);
+		}
 		g_free (contents);
 	} else {
 		g_printerr (_("Error reading file \"%s\": %s\n"), uri, gnome_vfs_result_to_string (result));
@@ -944,7 +970,7 @@ style_set_callback (GtkWidget *widget, GtkStyle  *previous_style, void *data)
 	client = gconf_client_get_default ();
 	color_scheme = gconf_client_get_int (client, "/apps/xchat/irc/color_scheme", NULL);
 	g_object_unref (client);
-	if (color_scheme == 0) {
+	if (color_scheme == 3) {
 		load_colors (color_scheme);
 		load_palette (color_scheme);
 	}
@@ -983,8 +1009,9 @@ conversation_panel_add_session (ConversationPanel *panel, struct session *sess,
 	g_hash_table_insert (panel->priv->buffers,            sess, buffer);
 	g_hash_table_insert (panel->priv->timestamp_notifies, sess, GINT_TO_POINTER (notify));
 
-	if (focus)
+	if (focus) {
 		conversation_panel_set_current (panel, sess);
+	}
 }
 
 void
@@ -995,6 +1022,10 @@ conversation_panel_set_current (ConversationPanel *panel, struct session *sess)
 	panel->priv->current = sess;
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
 	gtk_xtext_buffer_show (GTK_XTEXT (panel->priv->xtext), buffer, TRUE);
+
+#ifdef HAVE_LIBSEXY
+	conversation_panel_remove_tooltip (panel);
+#endif
 }
 
 void
@@ -1006,6 +1037,9 @@ conversation_panel_save_current (ConversationPanel *panel)
 	struct tm  date;
 	time_t     dtime;
 
+	if (!panel->priv->current)
+		return;
+
 	file_chooser = gtk_file_chooser_dialog_new (_("Save Transcript"),
 	                                            GTK_WINDOW (gui.main_window),
 	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -1069,8 +1103,9 @@ conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, g
 	int            leftlen;
 	unsigned char *tab;
 
-	if (len == 0)
+	if (len == 0) {
 		len = 1;
+	}
 
 	if (indent == FALSE) {
 		int     stamp_size;
@@ -1091,14 +1126,11 @@ conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, g
 	if (tab && tab < (text + len)) {
 		leftlen = tab - text;
 
-		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0)
-		{
+		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0) {
 			text = tab+1;
 			len -= leftlen;
 			gtk_xtext_append_indent (buffer, 0, 0, text, len);
-		}
-		else
-		{
+		} else {
 			strncpy (buffer->laststamp, text, leftlen);
 			buffer->laststamp[leftlen]=0;
 			gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
@@ -1116,6 +1148,9 @@ conversation_panel_print (ConversationPanel *panel, struct session *sess, guchar
 	int           len       = 0;
 
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
+	if (buffer == NULL) {
+		return;
+	}
 
 	/* split the text into separate lines */
 	while (1) {
@@ -1159,8 +1194,9 @@ conversation_panel_remove_session (ConversationPanel *panel, struct session *ses
 static void
 conversation_panel_lastlog_foreach (GtkXText *xtext, guchar *text, fe_lastlog_info *info)
 {
-	if (nocasestrstr (text, info->sstr))
+	if (nocasestrstr (text, info->sstr)) {
 		conversation_panel_print (info->panel, info->sess, text, prefs.indent_nicks);
+	}
 }
 
 void
@@ -1205,10 +1241,12 @@ conversation_panel_page_up (ConversationPanel *panel)
 	adj = GTK_XTEXT (panel->priv->xtext)->adj;
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value - (adj->page_size - 1);
-	if (value < 0)
+	if (value < 0) {
 		value = 0;
-	if (value > end)
+	}
+	if (value > end) {
 		value = end;
+	}
 	gtk_adjustment_set_value (adj, value);
 }
 
@@ -1221,18 +1259,19 @@ conversation_panel_page_down (ConversationPanel *panel)
 	adj = GTK_XTEXT (panel->priv->xtext)->adj;
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value + (adj->page_size - 1);
-	if (value < 0)
+	if (value < 0) {
 		value = 0;
-	if (value > end)
+	}
+	if (value > end) {
 		value = end;
+	}
 	gtk_adjustment_set_value (adj, value);
 }
 
 void
 conversation_panel_copy_selection (ConversationPanel *panel)
 {
-	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
-	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext), clipboard);
+	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext));
 }
 
 void
@@ -1259,11 +1298,25 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel
 	XEvent *xevent = (XEvent *)xev;
 
 	if (xevent->type == PropertyNotify) {
-		if (at == None)
+		if (at == None) {
 			at = XInternAtom (xevent->xproperty.display, "_XROOTPMAP_ID", True);
-		if (at == xevent->xproperty.atom)
+		}
+		if (at == xevent->xproperty.atom) {
 			redraw_transparency (panel);
+		}
 	}
 	return GDK_FILTER_CONTINUE;
 }
 #endif
+
+void
+conversation_panel_check_marker_visibility (ConversationPanel *panel)
+{
+	gtk_xtext_check_marker_visibility (GTK_XTEXT (panel->priv->xtext));
+}
+
+void
+conversation_panel_set_show_marker (ConversationPanel *panel, gboolean show_marker)
+{
+	gtk_xtext_set_show_marker (GTK_XTEXT (panel->priv->xtext), show_marker);
+}
