@@ -32,10 +32,12 @@
 #include "userlist-gui.h"
 #include "preferences.h"
 #include "setup-dialog.h"
+#include "migration.h"
 
 #include "conversation-panel.h"
 #include "status-bar.h"
 #include "topic-label.h"
+#include "connect-dialog.h"
 
 #include "palette.h"
 #include "preferences-page-plugins.h"
@@ -44,37 +46,46 @@
 #include "plugins.h"
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
+#include "../common/dcc.h"
 #include "../common/servlist.h"
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/cfgfiles.h"
 
-static gboolean opt_version = FALSE;
-static gboolean opt_noauto = FALSE;
-static gboolean opt_noplugins = FALSE;
-static gchar *opt_cfgdir = NULL;
+static gboolean  opt_version   = FALSE;
+static gboolean  opt_noplugins = FALSE;
+static gchar    *opt_cfgdir    = NULL;
 
 static GOptionEntry entries[] = {
 	{"cfgdir",     'd', 0, G_OPTION_ARG_FILENAME, &opt_cfgdir,           N_("Use directory instead of the default config dir"), "directory"},
 	{"no-auto",    'a', 0, G_OPTION_ARG_NONE,     &arg_dont_autoconnect, N_("Don't auto-connect to servers"),                   NULL},
 	{"no-plugins", 'n', 0, G_OPTION_ARG_NONE,     &opt_noplugins,        N_("Don't auto-load plugins"),                         NULL},
 	{"url",        'u', 0, G_OPTION_ARG_STRING,   &arg_url,              N_("Open an irc:// url"),                              "irc://server:port/channel"},
+	{"existing",   'e', 0, G_OPTION_ARG_NONE,     &arg_existing,         N_("Open URL in an existing xchat-gnome instance"),    NULL},
 	{"version",    'v', 0, G_OPTION_ARG_NONE,     &opt_version,          N_("Show version information"),                        NULL},
 	{ NULL }
 };
 
-static void setup_sm (gint argc, gchar *argv[]);
-static gint save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
-                          gint is_shutdown, GnomeInteractStyle interact_style,
-                          gint is_fast, gpointer client_data);
-static void kill_session (GnomeClient* client, gpointer client_data);
+static void     setup_sm        (gint                argc,
+                                 gchar              *argv[]);
+static gint     save_session    (GnomeClient        *client,
+                                 gint                phase,
+                                 GnomeSaveStyle      save_style,
+                                 gint                is_shutdown,
+                                 GnomeInteractStyle  interact_style,
+                                 gint                is_fast,
+                                 gpointer            client_data);
+static void     kill_session    (GnomeClient        *client,
+                                 gpointer            client_data);
+static gboolean not_autoconnect (void);
 
 int
 fe_args (int argc, char *argv[])
 {
 	GError *error = NULL;
 	GOptionContext *context;
-	GnomeProgram *program;
+
+	gui.main_window = NULL;
 
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -82,20 +93,27 @@ fe_args (int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
-	context = g_option_context_new ("");
+	context = g_option_context_new (NULL);
 
 #ifdef HAVE_LIBGNOME_214
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	program = gnome_program_init (PACKAGE, VERSION,
-	                              LIBGNOMEUI_MODULE, argc, argv,
-	                              GNOME_PARAM_GOPTION_CONTEXT, context,
-	                              GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
-	                              GNOME_PROGRAM_STANDARD_PROPERTIES,
-	                              NULL);
+	gui.program = gnome_program_init (PACKAGE, VERSION,
+	                                  LIBGNOMEUI_MODULE, argc, argv,
+	                                  GNOME_PARAM_GOPTION_CONTEXT, context,
+	                                  GNOME_PARAM_HUMAN_READABLE_NAME, "IRC Chat",
+	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
+	                                  NULL);
 #else
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+
+	gui.program = gnome_program_init (PACKAGE, VERSION,
+	                                  LIBGNOMEUI_MODULE, argc, argv,
+	                                  GNOME_PARAM_HUMAN_READABLE_NAME, "IRC Chat",
+	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
+	                                  NULL);
 #endif
 
 	if (error) {
@@ -104,15 +122,14 @@ fe_args (int argc, char *argv[])
 		return 0;
 	}
 
-	g_option_context_free (context);
-
 	if (opt_version) {
-		g_print ("xchat-gnome %s\n", VERSION);
+		g_print ("xchat-gnome %s\n", PACKAGE_VERSION);
 		return 0;
 	}
 
-	if (opt_cfgdir)
+	if (opt_cfgdir) {
 		xdir_fs = opt_cfgdir;
+	}
 
 	setup_sm (argc, argv);
 
@@ -122,11 +139,8 @@ fe_args (int argc, char *argv[])
 void
 fe_init (void)
 {
-	gchar *accel_map;
 	GConfClient *client;
 
-	gnome_vfs_init ();
-
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client, "/apps/xchat", GCONF_CLIENT_PRELOAD_NONE, NULL);
 	g_object_unref (client);
@@ -135,12 +149,14 @@ fe_init (void)
 	gui.quit = FALSE;
 	servlist_init ();
 	palette_init ();
+	run_migrations ();
+	if (!preferences_exist ()) {
+		run_setup_dialog ();
+	} else {
+		set_version ();
+	}
 	load_preferences ();
 	initialize_gui_1 ();
-	if (!preferences_exist ())
-		run_setup_dialog ();
-	else
-		set_version ();
 	initialize_gui_2 ();
 	run_main_window ();
 
@@ -148,20 +164,24 @@ fe_init (void)
 	prefs.use_server_tab = TRUE;
 	prefs.notices_tabs = FALSE;
 	prefs.servernotice = TRUE;
+	prefs.slist_skip = FALSE;
 
 	/* If we don't have a specific DCC IP address, force get-from-server */
-	if (strlen (prefs.dcc_ip_str) == 0)
+	if (strlen (prefs.dcc_ip_str) == 0) {
 		prefs.ip_from_server = TRUE;
+	}
 
 	/* Don't allow the core to autoload plugins. We use our own
 	 * method for autoloading.
 	 */
 	arg_skip_plugins = 1;
 
-	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
-	if (g_file_test (accel_map, G_FILE_TEST_EXISTS))
-		gtk_accel_map_load (accel_map);
-	g_free (accel_map);
+	if (not_autoconnect ()) {
+		ConnectDialog *cd;
+
+		cd = connect_dialog_new ();
+		gtk_widget_show_all (GTK_WIDGET (cd));
+	}
 
 #ifdef USE_PLUGIN
 	plugins_initialize ();
@@ -172,24 +192,26 @@ void
 fe_main (void)
 {
 	gtk_main ();
+
+#ifdef HAVE_LIBGNOME_214
+	g_object_unref (gui.program);
+#endif
+
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
 
 	/* FIXME: this is a crappy hack copied from fe-gtk. There's got
 	 * to be a way to ensure that the quit messages get sent before
 	 * we finish */
-	if (prefs.wait_on_exit)
+	if (prefs.wait_on_exit) {
 		sleep (3);
+	}
 }
 
 void
 fe_cleanup (void)
 {
-	gchar *accel_map;
-
-	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
-	gtk_accel_map_save (accel_map);
-	g_free (accel_map);
+	gnome_accelerators_sync ();
 }
 
 void
@@ -215,13 +237,26 @@ fe_new_window (struct session *sess, int focus)
 {
 	static gboolean loaded = FALSE;
 
-	gui.current_session = sess;
-	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
-	topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
-	if (sess->type == SESS_SERVER)
+	if (focus) {
+		gui.current_session = sess;
+		topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
+	}
+
+	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
+	if (sess->type == SESS_SERVER) {
 		navigation_tree_create_new_network_entry (gui.server_tree, sess);
-	else if (sess->type == SESS_CHANNEL || sess->type == SESS_DIALOG)
-		navigation_tree_create_new_channel_entry (gui.server_tree, sess);
+	} else if (sess->type == SESS_CHANNEL) {
+		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
+	} else if (sess->type == SESS_DIALOG) {
+		struct User *user = NULL;
+
+		user = userlist_find_global (sess->server, sess->channel);
+
+		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
+		if (user) {
+			topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, user->hostname);
+		}
+	}
 #ifdef USE_PLUGIN
 	if (!(opt_noplugins || loaded)) {
 		loaded = TRUE;
@@ -256,12 +291,15 @@ fe_input_add (int sok, int flags, void *func, void *data)
 
 	channel = g_io_channel_unix_new (sok);
 
-	if (flags & FIA_READ)
+	if (flags & FIA_READ) {
 		type |= G_IO_IN | G_IO_HUP | G_IO_ERR;
-	if (flags & FIA_WRITE)
+	}
+	if (flags & FIA_WRITE) {
 		type |= G_IO_OUT | G_IO_ERR;
-	if (flags & FIA_EX)
+	}
+	if (flags & FIA_EX) {
 		type |= G_IO_PRI;
+	}
 
 	tag = g_io_add_watch (channel, type, (GIOFunc) func, data);
 	g_io_channel_unref (channel);
@@ -291,12 +329,11 @@ void
 fe_set_hilight (struct session *sess)
 {
 	navigation_model_set_hilight (gui.tree_model, sess);
-	if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
-		gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+	fe_flash_window (sess);
 }
 
 void
-fe_set_tab_color (struct session *sess, int col, int flash)
+fe_set_tab_color (struct session *sess, int col)
 {
 	/* FIXME: implement */
 }
@@ -372,10 +409,11 @@ void
 fe_close_window (struct session *sess)
 {
 	if (!gui.quit) {
-		if (sess->type == SESS_CHANNEL)
+		if (sess->type == SESS_CHANNEL) {
 			navigation_tree_remove_channel (gui.server_tree, sess);
-		else
+		} else {
 			navigation_tree_remove_server (gui.server_tree, sess);
+		}
 	}
 
 	session_free (sess);
@@ -399,9 +437,11 @@ fe_print_text (struct session *sess, char *text)
 	conversation_panel_print (CONVERSATION_PANEL (gui.conversation_panel), sess, text, prefs.indent_nicks);
 	sess->new_data = TRUE;
 	navigation_model_set_hilight (gui.tree_model, sess);
-	if (sess->nick_said)
-		if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+	if (sess->nick_said) {
+		if (!gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
 			gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+		}
+	}
 }
 
 void
@@ -443,22 +483,58 @@ fe_userlist_clear (struct session *sess)
 void
 fe_dcc_add (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_add (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+		if (prefs.autodccchat == FALSE) {
+			GtkWidget *dialog;
+			gint response;
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (gui.main_window),
+			                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			                                 GTK_MESSAGE_QUESTION,
+			                                 GTK_BUTTONS_CANCEL,
+			                                 _("Incoming DCC Chat"));
+			gtk_dialog_add_button (GTK_DIALOG (dialog),
+			                       _("_Accept"),
+			                       GTK_RESPONSE_ACCEPT);
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+			                                          _("%s is attempting to create a direct chat. Do you wish to accept the connection?"),
+			                                          dcc->nick);
+
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
+			if (response == GTK_RESPONSE_ACCEPT) {
+				dcc_get (dcc);
+			} else {
+				dcc_abort (dcc->serv->server_session, dcc);
+			}
+			gtk_widget_destroy (dialog);
+		}
+	} else {
+		/* file transfers */
+		dcc_window_add (gui.dcc, dcc);
+	}
 }
 
 void
 fe_dcc_update (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_update (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+	} else {
+		/* file transfers */
+		dcc_window_update (gui.dcc, dcc);
+	}
 }
 
 void
 fe_dcc_remove (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_remove (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+	} else {
+		/* file transfers */
+		dcc_window_remove (gui.dcc, dcc);
+	}
 }
 
 int
@@ -575,8 +651,9 @@ fe_beep (void)
 }
 
 void
-fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
+fe_lastlog (session * sess, session * lastlog_sess, char *sstr, gboolean regexp)
 {
+	/* FIXME: handle regexp */
 	conversation_panel_lastlog (CONVERSATION_PANEL (gui.conversation_panel), sess, lastlog_sess, sstr);
 }
 
@@ -586,12 +663,14 @@ fe_set_lag (server *serv, int lag)
 	unsigned long now;
 	float seconds;
 
-	if (gui.quit)
+	if (gui.quit) {
 		return;
+	}
 
 	if (lag == -1) {
-		if (!serv->lag_sent)
+		if (!serv->lag_sent) {
 			return;
+		}
 		now = make_ping_time ();
 		seconds = (now - serv->lag_sent) / 1000000.0f;
 	} else {
@@ -604,8 +683,9 @@ fe_set_lag (server *serv, int lag)
 void
 fe_set_throttle (server * serv)
 {
-	if (gui.quit)
+	if (gui.quit) {
 		return;
+	}
 
 	status_bar_set_queue (STATUS_BAR (gui.status_bar), serv, serv->sendq_len);
 }
@@ -660,10 +740,12 @@ fe_gui_info (session * sess, int info_type)
 {
 	switch (info_type) {
 	case 0:
-		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (gui.main_window)))
+		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (gui.main_window))) {
 			return 2;
-		if (gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+		}
+		if (gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
 			return 1;
+		}
 		return 0;
 		break;
 	}
@@ -825,14 +907,16 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 void
 fe_open_url (const char *url)
 {
+	GdkScreen *screen;
 	GError *err = NULL;
 
+	screen = gtk_widget_get_screen (gui.main_window);
 	if (strstr (url, "://") == NULL) {
 		gchar *newword = g_strdup_printf ("http://%s", url);
-		gnome_url_show (newword, &err);
+		gnome_url_show_on_screen (newword, screen, &err);
 		g_free (newword);
 	} else {
-		gnome_url_show (url, &err);
+		gnome_url_show_on_screen (url, screen, &err);
 	}
 
 	if (err != NULL) {
@@ -849,8 +933,15 @@ fe_menu_del (menu_entry *entry)
 	/* FIXME: implement? */
 }
 
-void
+char *
 fe_menu_add (menu_entry *entry)
+{
+	/* FIXME: implement? */
+	return NULL;
+}
+
+void
+fe_set_color_paste (session *sess, int status)
 {
 	/* FIXME: implement? */
 }
@@ -870,7 +961,23 @@ fe_uselect (session *sess, char *word[], int do_clear, int scroll_to)
 void
 fe_server_event (server *serv, int type, int arg)
 {
-	/* FIXME: implement? */
+	GSList *list = sess_list;
+	session *sess;
+
+	while (list) {
+		sess = list->data;
+		if (sess->server == serv) {
+			switch (type) {
+			case FE_SE_LOGGEDIN:
+				if (arg == 0) {
+					/* No auto-join channels */
+					create_channel_list (sess);
+					break;
+				}
+			}
+		}
+		list = list->next;
+	}
 }
 
 void
@@ -879,6 +986,22 @@ fe_userlist_set_selected (struct session *sess)
 	/* FIXME: implement? */
 }
 
+void fe_flash_window (struct session *sess)
+{
+	if (!gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
+		gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+	}
+}
+
+void fe_get_file (const char  *title,
+                  char        *initial,
+                  void       (*callback) (void *userdata, char *file),
+                  void        *userdata,
+                  int          flags)
+{
+	/* FIXME: implement */
+}
+
 static void
 setup_sm (gint argc, gchar *argv[])
 {
@@ -903,10 +1026,9 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 	gint argc;
 	const gchar *prefix;
 
-	argv = g_new0 (gchar *, 3);
+	argv = g_new0 (gchar *, 2);
 	argv[0] = client_data;
-	argv[1] = "--no-auto";
-	argc = 2;
+	argc = 1;
 
 	gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
 	gnome_client_set_restart_command (client, argc, argv);
@@ -929,3 +1051,22 @@ kill_session (GnomeClient* client, gpointer client_data)
 	gui.quit = TRUE;
 	xchat_exit ();
 }
+
+static gboolean
+not_autoconnect (void)
+{
+	GSList *i;
+
+	if (arg_dont_autoconnect) {
+		return TRUE;
+	}
+
+	for (i = network_list; i; i = g_slist_next (i)) {
+		ircnet *net = (ircnet *) (i->data);
+		if (net->flags & FLAG_AUTO_CONNECT) {
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
