@@ -37,7 +37,6 @@
 #include "main-window.h"
 #include "navigation-tree.h"
 #include "palette.h"
-#include "preferences-dialog.h"
 #include "preferences.h"
 #include "userlist-gui.h"
 #include "util.h"
@@ -63,9 +62,7 @@ static void on_irc_quit_activate (GtkAction *action, gpointer data);
 static void on_edit_cut_activate (GtkAction *action, gpointer data);
 static void on_edit_copy_activate (GtkAction *action, gpointer data);
 static void on_edit_paste_activate (GtkAction *action, gpointer data);
-static void on_edit_clear_activate (GtkAction *action, gpointer data);
 static void on_edit_preferences_activate (GtkAction *action, gpointer data);
-static void on_network_information_activate (GtkAction *action, gpointer data);
 static void on_network_reconnect_activate (GtkAction *action, gpointer data);
 static void on_network_disconnect_activate (GtkAction *action, gpointer data);
 static void on_network_close_activate (GtkAction *action, gpointer data);
@@ -74,7 +71,6 @@ static void on_discussion_save_activate (GtkAction *action, gpointer data);
 static void on_discussion_leave_activate (GtkAction *action, gpointer data);
 static void on_discussion_close_activate (GtkAction *action, gpointer data);
 static void on_discussion_find_activate (GtkAction *action, gpointer data);
-static void on_discussion_clear_window_activate (GtkAction *action, gpointer data);
 static void on_discussion_bans_activate (GtkAction *action, gpointer data);
 static void on_discussion_topic_change_activate (GtkButton *widget, gpointer data);
 static void on_discussion_users_activate (GtkAction *action, gpointer data);
@@ -93,6 +89,9 @@ static gboolean on_resize (GtkWidget *widget, GdkEventConfigure *event, gpointer
 static gboolean on_hpane_move (GtkPaned *widget, GParamSpec *param_spec, gpointer data);
 
 static gboolean on_main_window_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data);
+static gboolean on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer data);
+
+static void nickname_style_set (GtkWidget *button, GtkStyle *previous_style, gpointer data);
 
 static GtkActionEntry action_entries [] = {
 
@@ -107,43 +106,40 @@ static GtkActionEntry action_entries [] = {
 	{ "PopupAction", NULL, "" },
 
 	/* IRC menu */
-	{ "IRCConnect",   GTK_STOCK_CONNECT, N_("_Connect"),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
-	{ "IRCDownloads", NULL,              N_("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
-	{ "IRCQuit",      GTK_STOCK_QUIT,    N_("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
+	{ "IRCConnect",   NULL,           N_("_Connect..."),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
+	{ "IRCDownloads", NULL,           N_("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
+	{ "IRCQuit",      GTK_STOCK_QUIT, N_("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
 
 	/* Edit menu */
 	{ "EditCut",         GTK_STOCK_CUT,         N_("Cu_t"),         "<control>X", NULL, G_CALLBACK (on_edit_cut_activate) },
 	{ "EditCopy",        GTK_STOCK_COPY,        N_("_Copy"),        "<control>C", NULL, G_CALLBACK (on_edit_copy_activate) },
 	{ "EditPaste",       GTK_STOCK_PASTE,       N_("_Paste"),       "<control>V", NULL, G_CALLBACK (on_edit_paste_activate) },
-	{ "EditClear",       GTK_STOCK_CLEAR,       N_("C_lear"),       "",           NULL, G_CALLBACK (on_edit_clear_activate) },
 	{ "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), "",           NULL, G_CALLBACK (on_edit_preferences_activate) },
 
 	/* Network menu */
-	{ "NetworkInformation", GTK_STOCK_DIALOG_INFO, N_("_Information"), "",                  NULL, G_CALLBACK (on_network_information_activate) },
 	{ "NetworkReconnect",   GTK_STOCK_REFRESH,     N_("_Reconnect"),   "<control>R",        NULL, G_CALLBACK (on_network_reconnect_activate) },
-	{ "NetworkDisconnect",  GTK_STOCK_STOP,        N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
+	{ "NetworkDisconnect",  GTK_STOCK_DISCONNECT,  N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
 	{ "NetworkClose",       GTK_STOCK_CLOSE,       N_("_Close"),       "<shift><control>W", NULL, G_CALLBACK (on_network_close_activate) },
-	{ "NetworkChannels",    GTK_STOCK_INDEX,       N_("_Channels..."), "<alt>C",            NULL, G_CALLBACK (on_network_channels_activate) },
+	{ "NetworkChannels",    NULL,                  N_("_Channels..."), "<alt>C",            NULL, G_CALLBACK (on_network_channels_activate) },
 
 	/* Discussion menu */
 	{ "DiscussionSave",        GTK_STOCK_SAVE,           N_("_Save Transcript"), "<control>S", NULL, G_CALLBACK (on_discussion_save_activate) },
 	{ "DiscussionLeave",       GTK_STOCK_QUIT,           N_("_Leave"),           "",           NULL, G_CALLBACK (on_discussion_leave_activate) },
 	{ "DiscussionClose",       GTK_STOCK_CLOSE,          N_("Cl_ose"),           "<control>W", NULL, G_CALLBACK (on_discussion_close_activate) },
 	{ "DiscussionFind",        GTK_STOCK_FIND,           N_("_Find"),            "<control>F", NULL, G_CALLBACK (on_discussion_find_activate) },
-	{ "DiscussionClearWindow", GTK_STOCK_CLEAR,          N_("_Clear Window"),    "<control>L", NULL, G_CALLBACK (on_discussion_clear_window_activate) },
-	{ "DiscussionChangeTopic", GTK_STOCK_REFRESH,        N_("Change _Topic"),    "<alt>T",     NULL, G_CALLBACK (on_discussion_topic_change_activate) },
+	{ "DiscussionChangeTopic", NULL,                     N_("Change _Topic"),    "<alt>T",     NULL, G_CALLBACK (on_discussion_topic_change_activate) },
 	{ "DiscussionBans",        GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "<alt>B",     NULL, G_CALLBACK (on_discussion_bans_activate) },
 	{ "DiscussionUsers",       NULL,                     N_("_Users"),           "<control>U", NULL, G_CALLBACK (on_discussion_users_activate) },
 
 	/* Go menu */
-	{ "GoPreviousNetwork",    GTK_STOCK_GOTO_FIRST, N_("Pre_vious Network"),    "<control>Up",   NULL, G_CALLBACK (on_go_previous_network_activate) },
-	{ "GoNextNetwork",        GTK_STOCK_GOTO_LAST,  N_("Nex_t Network"),        "<control>Down", NULL, G_CALLBACK (on_go_next_network_activate) },
-	{ "GoPreviousDiscussion", GTK_STOCK_GO_BACK,    N_("_Previous Discussion"), "<alt>Up",       NULL, G_CALLBACK (on_go_previous_discussion_activate) },
-	{ "GoNextDiscussion",     GTK_STOCK_GO_FORWARD, N_("_Next Discussion"),     "<alt>Down",     NULL, G_CALLBACK (on_go_next_discussion_activate) },
+	{ "GoPreviousNetwork",    NULL, N_("Pre_vious Network"),    "<control>Up",   NULL, G_CALLBACK (on_go_previous_network_activate) },
+	{ "GoNextNetwork",        NULL, N_("Nex_t Network"),        "<control>Down", NULL, G_CALLBACK (on_go_next_network_activate) },
+	{ "GoPreviousDiscussion", NULL, N_("_Previous Discussion"), "<alt>Up",       NULL, G_CALLBACK (on_go_previous_discussion_activate) },
+	{ "GoNextDiscussion",     NULL, N_("_Next Discussion"),     "<alt>Down",     NULL, G_CALLBACK (on_go_next_discussion_activate) },
 
 	/* Help menu */
-	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "",   NULL, G_CALLBACK (on_help_contents_activate) },
-	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    "",   NULL, G_CALLBACK (on_help_about_activate) },
+	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "F1", NULL, G_CALLBACK (on_help_contents_activate) },
+	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    NULL, NULL, G_CALLBACK (on_help_about_activate) },
 };
 
 void
@@ -151,12 +147,13 @@ initialize_main_window (void)
 {
 	GtkWidget *close, *menu_vbox, *widget;
 	GtkSizeGroup *group;
+	GtkAction *action;
+	gchar *path;
 
 	gui.main_window = glade_xml_get_widget (gui.xml, "xchat-gnome");
-	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",
-	                  G_CALLBACK (on_main_window_close), NULL);
-	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",
-	                  G_CALLBACK (on_main_window_focus_in), NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",    G_CALLBACK (on_main_window_close),     NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",  G_CALLBACK (on_main_window_focus_in),  NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "configure-event", G_CALLBACK (on_main_window_configure), NULL);
 
 	/* hook up the menus */
 	gui.action_group = gtk_action_group_new ("MenuAction");
@@ -170,22 +167,25 @@ initialize_main_window (void)
 	g_signal_connect (gui.manager, "add-widget", G_CALLBACK (on_add_widget), menu_vbox);
 
 	/* load the menus */
-	if (g_file_test ("../../data/xchat-gnome-ui.xml", G_FILE_TEST_EXISTS))
-		gtk_ui_manager_add_ui_from_file (gui.manager, "../../data/xchat-gnome-ui.xml", NULL);
-	else
-		gtk_ui_manager_add_ui_from_file (gui.manager, XCHATSHAREDIR "/xchat-gnome-ui.xml", NULL);
+	path = locate_data_file ("xchat-gnome-ui.xml");
+	gtk_ui_manager_add_ui_from_file (gui.manager, path, NULL);
+	g_free (path);
 
 	/* hook up accelerators */
 	gtk_window_add_accel_group (GTK_WINDOW (gui.main_window), gtk_ui_manager_get_accel_group (gui.manager));
 
 	close = glade_xml_get_widget (gui.xml, "close discussion");
-	g_signal_connect (G_OBJECT (close), "clicked", G_CALLBACK (on_discussion_close_activate), NULL);
+	g_signal_connect (close, "clicked", G_CALLBACK (on_discussion_close_activate), NULL);
 
-	gui.conversation_panel = glade_xml_get_widget (gui.xml, "conversation_panel");
-	gui.find_bar           = glade_xml_get_widget (gui.xml, "find_bar");
-	gui.status_bar         = glade_xml_get_widget (gui.xml, "status_bar");
-	gui.text_entry         = glade_xml_get_widget (gui.xml, "text_entry");
-	gui.topic_label        = glade_xml_get_widget (gui.xml, "topic_label");
+#define GW(name) ((gui.name) = glade_xml_get_widget (gui.xml, #name))
+	GW(conversation_panel);
+	GW(find_bar);
+	GW(status_bar);
+	GW(text_entry);
+	GW(topic_hbox);
+	GW(topic_label);
+	GW(nick_button);
+#undef GW
 
 	/* Hook up accelerators for pgup/pgdn */
 	{
@@ -260,13 +260,13 @@ initialize_main_window (void)
 		 */
 		closure = g_cclosure_new (G_CALLBACK (on_go_next_discussion_activate), NULL, NULL);
 		gtk_accel_group_connect (discussion_accel, GDK_Page_Down,
-				GDK_MOD1_MASK | GDK_CONTROL_MASK , GTK_ACCEL_VISIBLE, closure);
+		                         GDK_MOD1_MASK | GDK_CONTROL_MASK , GTK_ACCEL_VISIBLE, closure);
 
 		g_closure_unref (closure);
 
 		closure = g_cclosure_new (G_CALLBACK (on_go_previous_discussion_activate), NULL, NULL);
 		gtk_accel_group_connect (discussion_accel, GDK_Page_Up,
-				GDK_MOD1_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE, closure);
+		                        GDK_MOD1_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE, closure);
 
 		g_closure_unref (closure);
 
@@ -278,36 +278,46 @@ initialize_main_window (void)
 	group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 	gui.userlist_toggle = glade_xml_get_widget (gui.xml, "userlist_toggle");
 	g_signal_connect (G_OBJECT (gui.userlist_toggle), "toggled", G_CALLBACK (on_users_toggled), NULL);
-	if (g_file_test ("../../data/users.png", G_FILE_TEST_EXISTS))
-		gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_file ("../../data/users.png"));
-	else
-		gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_file (XCHATSHAREDIR "/users.png"));
+	path = locate_data_file ("users.png");
+	gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_file (path));
+	g_free (path);
 	gtk_size_group_add_widget (group, gui.userlist_toggle);
 	widget = glade_xml_get_widget (gui.xml, "entry hbox");
 	gtk_size_group_add_widget (group, widget);
 	g_object_unref (group);
 
 	/* connect nickname button */
-	widget = glade_xml_get_widget (gui.xml, "nickname");
-	gtk_button_set_use_underline (GTK_BUTTON (widget), FALSE);
-	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (on_nickname_clicked), NULL);
+	gtk_button_set_use_underline (GTK_BUTTON (gui.nick_button), FALSE);
+	g_signal_connect (G_OBJECT (gui.nick_button), "clicked",   G_CALLBACK (on_nickname_clicked), NULL);
+	g_signal_connect (G_OBJECT (GTK_BIN (gui.nick_button)->child), "style-set", G_CALLBACK (nickname_style_set),  NULL);
+
+	/* Temporarily disable menu items */
+	action = gtk_action_group_get_action (gui.action_group, "DiscussionBans");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 void
 run_main_window ()
 {
-	GtkWidget *pane, *widget;
+	GtkWidget *pane;
 	GConfClient *client;
 	int width, height;
+	int x, y;
 	int h;
 
 	client = gconf_client_get_default ();
 	width  = gconf_client_get_int (client, "/apps/xchat/main_window/width",  NULL);
 	height = gconf_client_get_int (client, "/apps/xchat/main_window/height", NULL);
-	if(width == 0 || height == 0)
+	if (width == 0 || height == 0) {
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), 800, 550);
-	else
+	} else {
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), width, height);
+	}
+	x = gconf_client_get_int (client, "/apps/xchat/main_window/x", NULL);
+	y = gconf_client_get_int (client, "/apps/xchat/main_window/y", NULL);
+	if (!(x == 0 || y == 0)) {
+		gtk_window_move (GTK_WINDOW (gui.main_window), x, y);
+	}
 	h = gconf_client_get_int (client, "/apps/xchat/main_window/hpane", NULL);
 	if(h != 0) {
 		GtkWidget *hpane = glade_xml_get_widget (gui.xml, "HPane");
@@ -319,12 +329,6 @@ run_main_window ()
 	g_object_unref (client);
 
 	gtk_widget_show (gui.main_window);
-
-	/* Temporarily disable menu items */
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/NetworkMenu/NetworkInformationItem");
-	gtk_widget_set_sensitive (widget, FALSE);
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionBansItem");
-	gtk_widget_set_sensitive (widget, FALSE);
 }
 
 void
@@ -333,10 +337,11 @@ rename_main_window (gchar *server, gchar *channel)
 	gchar *new_title;
 
 	if (server == NULL) {
-		if (channel && strlen (channel) != 0)
+		if (channel && strlen (channel) != 0) {
 			gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
-		else
+		} else {
 			gtk_window_set_title (GTK_WINDOW (gui.main_window), "XChat-GNOME");
+		}
 		return;
 	}
 	new_title = g_strconcat (server, ": ", channel, NULL);
@@ -357,10 +362,25 @@ on_irc_connect_activate (GtkAction *action, gpointer data)
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
+void
+save_main_window ()
+{
+	gint x, y;
+	GConfClient *client;
+
+	gtk_window_get_position (GTK_WINDOW (gui.main_window), &x, &y);
+	client = gconf_client_get_default ();
+	gconf_client_set_int (client, "/apps/xchat/main_window/x", x, NULL);
+	gconf_client_set_int (client, "/apps/xchat/main_window/y", y, NULL);
+	g_object_unref (client);
+}
+
 static void
 on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
+	save_main_window ();
 	gui.quit = TRUE;
+
 	gtk_widget_hide (GTK_WIDGET (gui.dcc));
 	userlist_gui_hide ();
 	xchat_exit ();
@@ -369,6 +389,7 @@ on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data)
 static void
 on_irc_quit_activate (GtkAction *action, gpointer data)
 {
+	save_main_window ();
 	gtk_widget_hide (GTK_WIDGET (gui.main_window));
 	gtk_widget_hide (GTK_WIDGET (gui.dcc));
 	userlist_gui_hide ();
@@ -379,41 +400,37 @@ on_irc_quit_activate (GtkAction *action, gpointer data)
 static void
 on_edit_cut_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_cut_clipboard(GTK_EDITABLE(gui.text_entry));
+	gtk_editable_cut_clipboard (GTK_EDITABLE (gui.text_entry));
 }
 
 static void
 on_edit_copy_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_copy_clipboard(GTK_EDITABLE(gui.text_entry));
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (gui.text_entry), NULL, NULL)) {
+		/* There is something selected in the text_entry */
+		gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
+	} else {
+		/* Nothing selected, we copy from the conversation panel */
+		conversation_panel_copy_selection (CONVERSATION_PANEL (gui.conversation_panel));
+	}
 }
 
 static void
 on_edit_paste_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_paste_clipboard(GTK_EDITABLE(gui.text_entry));
-}
-
-static void
-on_edit_clear_activate (GtkAction *action, gpointer data)
-{
-	gtk_editable_delete_selection(GTK_EDITABLE(gui.text_entry));
+	gtk_editable_paste_clipboard (GTK_EDITABLE (gui.text_entry));
 }
 
 static void
 on_edit_preferences_activate (GtkAction *action, gpointer data)
 {
-	PreferencesDialog *dialog;
+	if (!gui.prefs_dialog) {
+		gui.prefs_dialog = preferences_dialog_new ();
+		g_object_add_weak_pointer (G_OBJECT (gui.prefs_dialog),
+		                           (gpointer *) (&gui.prefs_dialog));
+	}
 
-	dialog = preferences_dialog_new ();
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
-static void
-on_network_information_activate (GtkAction *action, gpointer data)
-{
-	/* FIXME: implement */
+	preferences_dialog_show (gui.prefs_dialog);
 }
 
 static void
@@ -426,8 +443,9 @@ static void
 on_network_disconnect_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s)
+	if (s) {
 		s->server->disconnect (s, TRUE, -1);
+	}
 }
 
 static void
@@ -479,14 +497,15 @@ static void
 on_discussion_leave_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s->type == SESS_CHANNEL) {
+	if ((s != NULL) && (s->type == SESS_CHANNEL) && (s->channel[0] != '\0')) {
 		gchar *text;
 		GConfClient *client;
 
 		client = gconf_client_get_default ();
 		text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
-		if (text == NULL)
+		if (text == NULL) {
 			text = g_strdup (_("Ex-Chat"));
+		}
 		s->server->p_part (s->server, s->channel, text);
 		g_object_unref (client);
 		g_free (text);
@@ -497,20 +516,8 @@ static void
 on_discussion_close_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s == NULL)
+	if (s == NULL) {
 		return;
-/*	navigation_tree_select_next_channel (gui.server_tree, TRUE);*/
-	if (s->type == SESS_CHANNEL) {
-		gchar *text;
-		GConfClient *client;
-
-		client = gconf_client_get_default ();
-		text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
-		if (text == NULL)
-			text = g_strdup (_("Ex-Chat"));
-		s->server->p_part (s->server, s->channel, text);
-		g_object_unref (client);
-		g_free (text);
 	}
 	fe_close_window (s);
 	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
@@ -524,12 +531,6 @@ on_discussion_find_activate (GtkAction *action, gpointer data)
 	find_bar_open (FIND_BAR (gui.find_bar));
 }
 
-static void
-on_discussion_clear_window_activate (GtkAction *action, gpointer data)
-{
-	conversation_panel_clear (CONVERSATION_PANEL (gui.conversation_panel), gui.current_session);
-}
-
 static void
 on_discussion_bans_activate (GtkAction *action, gpointer data)
 {
@@ -597,50 +598,60 @@ on_help_about_activate (GtkAction *action, gpointer data)
 }
 
 static void
-nickname_get_str_response (GtkDialog *dialog, gint arg1, gpointer entry)
+nickname_dialog_entry_activated (GtkEntry *entry, GtkDialog *dialog)
 {
-	gchar *text, *buf;
-	GtkWidget *check;
-
-	if (arg1 == GTK_RESPONSE_ACCEPT || arg1 == GTK_RESPONSE_OK) {
-		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
-		check = glade_xml_get_widget (gui.xml, "nickname dialog check");
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)))
-			buf = g_strdup_printf ("allserv nick %s", text);
-		else
-			buf = g_strdup_printf ("nick %s", text);
-		handle_command (current_sess, buf, FALSE);
-		g_free (buf);
-	}
-	gtk_widget_hide_all (GTK_WIDGET (dialog));
-}
-
-static void
-nickname_str_enter (GtkWidget *entry, GtkWidget *dialog)
-{
-	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
 static void
 on_nickname_clicked (GtkButton *widget, gpointer user_data)
 {
 	GtkWidget *dialog;
-	GtkWidget *entry;
+	GtkWidget *entry, *away;
+	gint result;
 
-	if (gui.current_session == NULL)
+	if (gui.current_session == NULL) {
 		return;
+	}
 	current_sess = gui.current_session;
 
 	dialog = glade_xml_get_widget (gui.xml, "nickname dialog");
 	entry = glade_xml_get_widget (gui.xml, "nickname dialog entry");
+	away = glade_xml_get_widget (gui.xml, "nickname dialog away");
 
 	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick);
-	g_signal_connect (G_OBJECT (entry), "activate",
-	                  G_CALLBACK (nickname_str_enter), dialog);
-	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (nickname_get_str_response), entry);
-	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (away), current_sess->server->is_away);
+	g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (nickname_dialog_entry_activated), dialog);
 
-	gtk_widget_show_all (dialog);
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (result == GTK_RESPONSE_OK) {
+		GtkWidget *check_all;
+		gboolean all;
+		gchar *text, *buf;
+
+		check_all = glade_xml_get_widget (gui.xml, "nickname dialog all");
+		all = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check_all));
+
+		/* Nick */
+		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+		if (all) {
+			buf = g_strdup_printf ("allserv nick %s", text);
+		} else {
+			buf = g_strdup_printf ("nick %s", text);
+		}
+		handle_command (current_sess, buf, FALSE);
+		g_free (buf);
+
+		/* Away */
+		if (current_sess->server->is_away != gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (away))) {
+			if (all) {
+				handle_command (current_sess, "allserv away", FALSE);
+			} else {
+				handle_command (current_sess, "away", FALSE);	
+			}
+		}
+	}
+	gtk_widget_hide (dialog);
 }
 
 static gboolean
@@ -681,21 +692,35 @@ on_users_toggled (GtkToggleButton *widget, gpointer user_data)
 
 	toggled = gtk_toggle_button_get_active (widget);
 
-	if (toggled)
+	if (toggled) {
 		userlist_gui_show ();
-	else
+	} else {
 		userlist_gui_hide ();
+	}
 }
 
 void
 set_nickname (struct server *serv, char *newnick)
 {
+	GtkLabel *label;
+
+	if (gui.current_session == NULL) {
+		return;
+	}
+
+	label = GTK_LABEL (GTK_BIN (gui.nick_button)->child);
+
+	if (serv == NULL) {
+		gtk_label_set_text (label, "");
+		return;
+	}
+
 	if (serv == gui.current_session->server) {
-		GtkWidget *nick = glade_xml_get_widget (gui.xml, "nickname");
-		if (newnick == NULL)
-			gtk_button_set_label (GTK_BUTTON (nick), serv->nick);
-		else
-			gtk_button_set_label (GTK_BUTTON (nick), newnick);
+		if (newnick == NULL) {
+			gtk_label_set_text (label, serv->nick);
+		} else {
+			gtk_label_set_text (label, newnick);
+		}
 		set_nickname_color (serv);
 	}
 }
@@ -703,21 +728,28 @@ set_nickname (struct server *serv, char *newnick)
 void
 set_nickname_color (struct server *serv)
 {
+	if (gui.current_session == NULL) {
+		return;
+	}
+
 	if (serv == gui.current_session->server) {
-		GtkWidget *nick_button;
 		GtkLabel *label;
 		PangoAttribute *attr;
 		PangoAttrList *l;
+		GtkStyle *style;
+		GdkColor *color;
 
 		l = pango_attr_list_new ();
-		nick_button = glade_xml_get_widget (gui.xml, "nickname");
-		label = GTK_LABEL (GTK_BIN (nick_button)->child);
+		label = GTK_LABEL (GTK_BIN (gui.nick_button)->child);
 
-		if (serv->is_away)
-			attr = pango_attr_foreground_new (0x9999, 0x9999, 0x9999);
-		else
-			/* FIXME: maybe we should get the color from GTK theme */
-			attr = pango_attr_foreground_new (0x0000, 0x0000, 0x0000);
+		style = gtk_widget_get_style (GTK_WIDGET (label));
+
+		if (serv->is_away) {
+			color = &(style->fg[GTK_STATE_INSENSITIVE]);
+		} else {
+			color = &(style->fg[GTK_STATE_NORMAL]);
+		}
+		attr = pango_attr_foreground_new (color->red, color->green, color->blue);
 
 		attr->start_index = 0;
 		attr->end_index = G_MAXUINT;
@@ -731,6 +763,24 @@ set_nickname_color (struct server *serv)
 static gboolean
 on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 {
+	conversation_panel_check_marker_visibility (CONVERSATION_PANEL (gui.conversation_panel));
 	gtk_window_set_urgency_hint (GTK_WINDOW (widget), FALSE);
 	return FALSE;
 }
+
+static gboolean
+on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
+{
+	conversation_panel_queue_tdraw (CONVERSATION_PANEL (gui.conversation_panel));
+	return FALSE;
+}
+
+static void
+nickname_style_set (GtkWidget *button, GtkStyle *previous_style, gpointer data)
+{
+	if (gui.current_session == NULL) {
+		return;
+	}
+
+	set_nickname_color (gui.current_session->server);
+}
