@@ -1,7 +1,7 @@
 /*
  * preferences-page-colors.c - helpers for the colors preferences page
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,17 +28,31 @@
 #include "preferences-page-colors.h"
 #include "preferences-dialog.h"
 #include "xtext.h"
+#include "util.h"
 
 static int scheme;
 
+static void
+show_colors_changed (GtkToggleButton *button, const gchar *key)
+{
+	GConfClient *client;
+	gboolean value;
+
+	client = gconf_client_get_default ();
+	value = gtk_toggle_button_get_active (button);
+	gconf_client_set_bool (client, key, value, NULL);
+	g_object_unref (client);
+}
+
 static void
 color_button_changed (GtkColorButton *button, gpointer data)
 {
 	int index = GPOINTER_TO_INT (data);
 	GdkColor c;
 
-	if (scheme != 2)
+	if (scheme != 2) {
 		return;
+	}
 
 	gtk_color_button_get_color (button, &c);
 	if (index < 32) {
@@ -100,24 +114,26 @@ colors_changed (GtkComboBox *combo_box, PreferencesColorsPage *page)
 {
 	int i, selection;
 	GConfClient *client;
+	gboolean sensitive;
 
 	client = gconf_client_get_default ();
 
 	selection = gtk_combo_box_get_active (combo_box);
 	scheme = selection;
 
+	sensitive = (selection == 2);
+
 	/* If we've set custom, sensitize the color buttons */
-	if (selection == 2) {
-		for (i = 0; i < 4; i++)
-			gtk_widget_set_sensitive (page->color_buttons[i], TRUE);
-		for (i = 0; i < 32; i++)
-			gtk_widget_set_sensitive (page->palette_buttons[i], TRUE);
-	} else {
-		for (i = 0; i < 4; i++)
-			gtk_widget_set_sensitive (page->color_buttons[i], FALSE);
-		for (i = 0; i < 32; i++)
-			gtk_widget_set_sensitive (page->palette_buttons[i], FALSE);
+	for (i = 0; i < 4; i++) {
+		gtk_widget_set_sensitive (page->color_buttons[i], sensitive);
 	}
+	for (i = 0; i < 32; i++) {
+		gtk_widget_set_sensitive (page->palette_buttons[i], sensitive);
+	}
+
+	gtk_widget_set_sensitive (page->mirc_colors_box, sensitive);
+	gtk_widget_set_sensitive (page->extra_colors_box, sensitive);
+
 	gconf_client_set_int (client, "/apps/xchat/irc/color_scheme", selection, NULL);
 	set_color_buttons (selection, page->color_buttons);
 	set_palette_buttons (selection, page->palette_buttons);
@@ -125,6 +141,17 @@ colors_changed (GtkComboBox *combo_box, PreferencesColorsPage *page)
 	g_object_unref (client);
 }
 
+static void
+gconf_show_colors_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, GtkToggleButton *button)
+{
+	gboolean toggle;
+
+	g_signal_handlers_block_by_func (button, "toggled", show_colors_changed);
+	toggle = gconf_client_get_bool (client, entry->key, NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), toggle);
+	g_signal_handlers_unblock_by_func (button, "toggled", show_colors_changed);
+}
+
 static void
 gconf_color_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, PreferencesColorsPage *page)
 {
@@ -145,10 +172,13 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	GtkSizeGroup *group;
 	GtkTreeIter iter;
 	gint i, j;
+	gchar *path;
+	gboolean toggle;
 
 	palette_init ();
 
 #define GW(name) ((page->name) = glade_xml_get_widget (xml, #name))
+	GW(show_colors);
 	GW(color_label_1);
 	GW(color_label_2);
 	GW(color_label_3);
@@ -161,7 +191,9 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	GW(foreground_mark_hbox);
 	GW(background_mark_hbox);
 
+	GW(mirc_colors_box);
 	GW(mirc_palette_table);
+	GW(extra_colors_box);
 	GW(extra_palette_table);
 #undef GW
 
@@ -178,7 +210,6 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 			gint c = j * 8 + i;
 			page->palette_buttons[c] = gtk_color_button_new ();
 			gtk_widget_show (page->palette_buttons[c]);
-			gtk_widget_set_sensitive (page->palette_buttons[c], FALSE);
 			gtk_table_attach_defaults (GTK_TABLE (page->mirc_palette_table), page->palette_buttons[c], i, i+1, j, j+1);
 			gtk_color_button_set_color (GTK_COLOR_BUTTON (page->palette_buttons[c]), &colors[c]);
 			g_signal_connect (G_OBJECT (page->palette_buttons[c]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (c));
@@ -189,7 +220,6 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 			gint c = j * 8 + i + 16;
 			page->palette_buttons[c] = gtk_color_button_new ();
 			gtk_widget_show (page->palette_buttons[c]);
-			gtk_widget_set_sensitive (page->palette_buttons[c], FALSE);
 			gtk_table_attach_defaults (GTK_TABLE (page->extra_palette_table), page->palette_buttons[c], i, i+1, j, j+1);
 			gtk_color_button_set_color (GTK_COLOR_BUTTON (page->palette_buttons[c]), &colors[c]);
 			g_signal_connect (G_OBJECT (page->palette_buttons[c]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (c));
@@ -199,7 +229,6 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	for (i = 0; i < 4; i++) {
 		page->color_buttons[i] = gtk_color_button_new ();
 		gtk_widget_show (page->color_buttons[i]);
-		gtk_widget_set_sensitive (page->color_buttons[i], FALSE);
 	}
 	gtk_box_pack_start (GTK_BOX (page->text_color_hbox),       page->color_buttons[0], FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (page->background_color_hbox), page->color_buttons[1], FALSE, TRUE, 0);
@@ -211,15 +240,16 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_label_set_mnemonic_widget (GTK_LABEL (page->color_label_4), page->color_buttons[2]);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (page->color_label_5), page->color_buttons[3]);
 
+	g_signal_connect (G_OBJECT (page->show_colors),      "toggled",   G_CALLBACK (show_colors_changed),  "/apps/xchat/irc/showcolors");
 	g_signal_connect (G_OBJECT (page->color_buttons[0]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (32));
 	g_signal_connect (G_OBJECT (page->color_buttons[1]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (33));
 	g_signal_connect (G_OBJECT (page->color_buttons[2]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (34));
 	g_signal_connect (G_OBJECT (page->color_buttons[3]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (35));
 
-	if (g_file_test ("../../data/color.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/color.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/color.png", NULL);
+	path = locate_data_file ("color.png");
+	page->icon = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
+
 	gtk_list_store_append (p->page_store, &iter);
 	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Colors"), 2, 1, -1);
 
@@ -227,13 +257,19 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_combo_box_append_text (GTK_COMBO_BOX (page->combo), _("Black on White"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (page->combo), _("White on Black"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (page->combo), _("Custom"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (page->combo), _("System Theme Colors"));
 	gtk_widget_show (page->combo);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (page->color_label_1), page->combo);
 	gtk_box_pack_start (GTK_BOX (page->foreground_background_hbox), page->combo, FALSE, TRUE, 0);
 	scheme = gconf_client_get_int (p->gconf, "/apps/xchat/irc/color_scheme", NULL);
 
-	page->notify = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/color_scheme",
-	                                        (GConfClientNotifyFunc) gconf_color_changed, page, NULL, NULL);
+	page->nh[0] = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/showcolors",
+	                                       (GConfClientNotifyFunc) gconf_show_colors_changed, page->show_colors, NULL, NULL);
+	page->nh[1] = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/color_scheme",
+	                                       (GConfClientNotifyFunc) gconf_color_changed, page, NULL, NULL);
+
+	toggle = gconf_client_get_bool (p->gconf, "/apps/xchat/irc/showcolors", NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->show_colors), toggle);
 
 	g_signal_connect (G_OBJECT (page->combo), "changed", G_CALLBACK (colors_changed), page);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (page->combo), scheme);
@@ -244,10 +280,13 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 void
 preferences_page_colors_free (PreferencesColorsPage *page)
 {
+	gint i;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
-	gconf_client_notify_remove (client, page->notify);
+	for (i = 0; i < 2; i++) {
+		gconf_client_notify_remove (client, page->nh[i]);
+	}
 	g_object_unref (client);
 	g_object_unref (page->icon);
 	g_free (page);
