@@ -1,7 +1,7 @@
 /*
  * preferences-page-effects.c - helpers for the effects preferences page
  *
- * Copyright (C) 2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 
 #include "preferences-page-effects.h"
 #include "preferences-dialog.h"
+#include "util.h"
 
 static void
 type_changed (GtkToggleButton *button, PreferencesEffectsPage *page)
@@ -143,8 +144,9 @@ update_preview (GtkFileChooser *file_chooser, PreferencesEffectsPage *page)
 		g_free (filename);
 
 		gtk_image_set_from_pixbuf (GTK_IMAGE (page->image_preview), pixbuf);
-		if (pixbuf)
+		if (pixbuf) {
 			g_object_unref (pixbuf);
+		}
 	}
 
 	gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
@@ -157,7 +159,7 @@ preferences_page_effects_new (gpointer prefs_dialog, GladeXML *xml)
 	PreferencesDialog *p = (PreferencesDialog *) prefs_dialog;
 	GtkTreeIter iter;
 	int type;
-	char *filename;
+	gchar *filename, *path;
 	float transparency;
 
 #define GW(name) ((page->name) = glade_xml_get_widget (xml, #name))
@@ -168,10 +170,10 @@ preferences_page_effects_new (gpointer prefs_dialog, GladeXML *xml)
 	GW(background_transparency);
 #undef GW
 
-	if (g_file_test ("../../data/effects.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/effects.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/effects.png", NULL);
+	path = locate_data_file ("effects.png");
+	page->icon = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
+
 	gtk_list_store_append (p->page_store, &iter);
 	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Effects"), 2, 2, -1);
 
@@ -223,8 +225,9 @@ preferences_page_effects_free (PreferencesEffectsPage *page)
 	gint i;
 
 	client = gconf_client_get_default ();
-	for (i = 0; i < 3; i++)
+	for (i = 0; i < 3; i++) {
 		gconf_client_notify_remove (client, page->nh[i]);
+	}
 	g_object_unref (client);
 
 	g_object_unref (page->icon);
