@@ -23,6 +23,9 @@
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 #include <string.h>
+#ifdef HAVE_LIBSEXY
+#include <libsexy/sexy-tree-view.h>
+#endif
 
 #include "preferences-page-plugins.h"
 #include "preferences-dialog.h"
@@ -43,7 +46,10 @@ extern GSList *plugin_list; 	// xchat's list of loaded plugins.
 extern GSList *enabled_plugins;	// Our list of loaded plugins.
 extern XChatGUI gui;
 
-static PreferencesPluginsPage *pageref;
+static PreferencesPagePlugins *pageref;
+#ifdef HAVE_LIBSEXY
+static GtkWidget* get_plugin_tooltip (SexyTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, PreferencesPagePlugins *page);
+#endif
 
 enum
 {
@@ -55,6 +61,8 @@ enum
 	COL_DISPLAY
 };
 
+static GObjectClass *parent_class;
+
 static void
 fe_plugin_add (char *filename)
 {
@@ -90,8 +98,9 @@ fe_plugin_add (char *filename)
 			    COL_DESC, desc, COL_FILENAME, filename, COL_DISPLAY, display, -1);
 	g_free (display);
 
-	if (handle != NULL)
+	if (handle != NULL) {
 		g_module_close (handle);
+	}
 }
 
 static gint
@@ -101,7 +110,7 @@ filename_test (gconstpointer a, gconstpointer b)
 }
 
 static void
-load_unload (char *filename, gboolean loaded, PreferencesPluginsPage *page, GtkTreeIter iter)
+load_unload (char *filename, gboolean loaded, PreferencesPagePlugins *page, GtkTreeIter iter)
 {
 	GConfClient *client;
 
@@ -122,7 +131,7 @@ load_unload (char *filename, gboolean loaded, PreferencesPluginsPage *page, GtkT
 			}
 
 		} else {
-			gchar *errmsg = g_strdup_printf (_("An error occured unloading %s"), filename);
+			gchar *errmsg = g_strdup_printf (_("An error occurred unloading %s"), filename);
 			error_dialog (_("Plugin Unload Failed"), errmsg);
 		}
 
@@ -132,7 +141,6 @@ load_unload (char *filename, gboolean loaded, PreferencesPluginsPage *page, GtkT
 		if (err == NULL) {
 			gtk_list_store_set (page->plugin_store, &iter, COL_LOADED, TRUE, -1);
 			enabled_plugins = g_slist_append (enabled_plugins, filename);
-
 		} else {
 			error_dialog (_("Plugin Load Failed"), err);
 		}
@@ -145,7 +153,7 @@ load_unload (char *filename, gboolean loaded, PreferencesPluginsPage *page, GtkT
 }
 
 static void
-load_toggled (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesPluginsPage *page)
+load_toggled (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesPagePlugins *page)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -165,7 +173,7 @@ load_toggled (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesPluginsP
 }
 
 static void
-open_plugin_clicked (GtkButton *button, PreferencesPluginsPage *page)
+open_plugin_clicked (GtkButton *button, PreferencesPagePlugins *page)
 {
 	GtkWidget *file_selector;
 	const gchar *homedir;
@@ -191,7 +199,7 @@ open_plugin_clicked (GtkButton *button, PreferencesPluginsPage *page)
 }
 
 static void
-remove_plugin_clicked (GtkButton *button, PreferencesPluginsPage *page)
+remove_plugin_clicked (GtkButton *button, PreferencesPagePlugins *page)
 {
 	GtkTreeSelection *select;
 	GtkTreeIter iter;
@@ -209,13 +217,13 @@ remove_plugin_clicked (GtkButton *button, PreferencesPluginsPage *page)
 }
 
 static void
-selection_changed (GtkTreeSelection *select, PreferencesPluginsPage *page)
+selection_changed (GtkTreeSelection *select, PreferencesPagePlugins *page)
 {
 	gtk_widget_set_sensitive (page->plugins_remove, gtk_tree_selection_get_selected (select, NULL, NULL));
 }
 
 static void
-row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, PreferencesPluginsPage *page)
+row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, PreferencesPagePlugins *page)
 {
 	GtkTreeSelection *select;
 	GtkTreeModel *model;
@@ -249,29 +257,60 @@ set_loaded_if_match (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
 	return FALSE;
 }
 
-PreferencesPluginsPage *
+#ifdef HAVE_LIBSEXY
+static GtkWidget*
+get_plugin_tooltip (SexyTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, PreferencesPagePlugins *page)
+{
+	GtkTreeIter iter;
+	GtkWidget *label;
+	gchar *text;
+
+	label = NULL;
+
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (page->plugin_store), &iter, path)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (page->plugin_store), &iter, COL_DESC, &text, -1);
+		label = gtk_label_new (text);
+		g_free (text);
+		gtk_widget_show_all (label);
+	}
+
+	return label;
+}
+#endif
+
+PreferencesPagePlugins*
 preferences_page_plugins_new (gpointer prefs_dialog, GladeXML *xml)
 {
-	PreferencesPluginsPage *page = g_new0 (PreferencesPluginsPage, 1);
+	PreferencesPagePlugins *page = g_object_new (preferences_page_plugins_get_type (), NULL);
 	PreferencesDialog *p = (PreferencesDialog *) prefs_dialog;
 	GtkTreeIter iter;
 	GtkTreeSelection *select;
+	GtkWidget *scroll;
 	GSList *list;
 	xchat_plugin *plugin;
 
 	const gchar *homedir;
-	gchar *xchatdir;
+	gchar *xchatdir, *path;
 
 #define GW(name) ((page->name) = glade_xml_get_widget (xml, #name))
-	GW(plugins_list);
 	GW(plugins_open);
 	GW(plugins_remove);
 #undef GW
 
-	if (g_file_test ("../../data/plugin-manager.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/plugin-manager.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/plugin-manager.png", NULL);
+#ifdef HAVE_LIBSEXY
+	page->plugins_list = sexy_tree_view_new ();
+#else
+	page->plugins_list = gtk_tree_view_new ();
+#endif
+	scroll = glade_xml_get_widget (xml, "scrolledwindow19");
+	gtk_container_add (GTK_CONTAINER (scroll), page->plugins_list);
+
+	gtk_widget_show (page->plugins_list);
+
+	path = locate_data_file ("plugin-manager.png");
+	page->icon = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
+
 	gtk_list_store_append (p->page_store, &iter);
 	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Scripts and Plugins"), 2, 5, -1);
 
@@ -303,6 +342,10 @@ preferences_page_plugins_new (gpointer prefs_dialog, GladeXML *xml)
 	g_signal_connect (G_OBJECT (select),               "changed",       G_CALLBACK (selection_changed),     page);
 	g_signal_connect (G_OBJECT (page->plugins_list),   "row-activated", G_CALLBACK (row_activated),         page);
 
+#ifdef HAVE_LIBSEXY
+	g_signal_connect (G_OBJECT (page->plugins_list),   "get-tooltip",   G_CALLBACK (get_plugin_tooltip),    page);
+#endif
+
 	homedir = g_get_home_dir ();
 	xchatdir = g_strdup_printf ("%s/.xchat2/plugins", homedir);
 	pageref = page;
@@ -333,10 +376,50 @@ preferences_page_plugins_new (gpointer prefs_dialog, GladeXML *xml)
 	return page;
 }
 
-void
-preferences_page_plugins_free (PreferencesPluginsPage *page)
+static void
+preferences_page_plugins_dispose (GObject *object)
 {
-	g_object_unref (page->icon);
-	g_object_unref (page->plugin_store);
-	g_free (page);
+	PreferencesPagePlugins *page = (PreferencesPagePlugins *) object;
+
+	if (page->icon) {
+		g_object_unref (page->icon);
+		page->icon = NULL;
+	}
+
+	if (page->plugin_store) {
+		g_object_unref (page->plugin_store);
+		page->plugin_store = NULL;
+	}
+
+	parent_class->dispose (object);
+}
+
+static void
+preferences_page_plugins_class_init (PreferencesPagePluginsClass *klass)
+{
+	GObjectClass *object_class = (GObjectClass *) klass;
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->dispose = preferences_page_plugins_dispose;
+}
+
+GType
+preferences_page_plugins_get_type (void)
+{
+	static GType preferences_page_plugins_type = 0;
+	if (G_UNLIKELY (preferences_page_plugins_type == 0)) {
+		static const GTypeInfo preferences_page_plugins_info = {
+			sizeof (PreferencesPagePluginsClass),
+			NULL, NULL,
+			(GClassInitFunc) preferences_page_plugins_class_init,
+			NULL, NULL,
+			sizeof (PreferencesPagePlugins),
+			0,
+			NULL,
+		};
+		preferences_page_plugins_type = g_type_register_static (G_TYPE_OBJECT, "PreferencesPagePlugins", &preferences_page_plugins_info, 0);
+	}
+
+	return preferences_page_plugins_type;
 }
