@@ -22,9 +22,11 @@
 #include <config.h>
 #include <glib/gi18n.h>
 #include <string.h>
+#include <gtk/gtk.h>
 #include "userlist.h"
 #include "palette.h"
 #include "pixmaps.h"
+#include "gui.h"
 
 static void userlist_class_init (UserlistClass *klass);
 static void userlist_init       (Userlist *userlist);
@@ -124,8 +126,9 @@ get_user_icon (struct server *serv, struct User *user)
 	char *pre;
 	int level;
 
-	if (!user)
+	if (!user) {
 		return NULL;
+	}
 
 	switch (user->prefix[0]) {
 	case '\0': return NULL;
@@ -148,8 +151,9 @@ get_user_icon (struct server *serv, struct User *user)
 				break; /* 3+, no icons */
 			}
 			level++;
-			if (pre == serv->nick_prefixes)
+			if (pre == serv->nick_prefixes) {
 				break;
+			}
 			pre--;
 		}
 	}
@@ -164,8 +168,9 @@ userlist_insert (Userlist *userlist, session *sess, struct User *newuser, int ro
 	GdkPixbuf *icon;
 	GList *item;
 
-	if (!store)
+	if (!store) {
 		store = create_userlist (userlist, sess);
+	}
 
 	icon = get_user_icon (sess->server, newuser);
 
@@ -175,6 +180,8 @@ userlist_insert (Userlist *userlist, session *sess, struct User *newuser, int ro
 	item = g_list_append (NULL, newuser->nick);
 	g_completion_add_items (store->completion, item);
 	store->completion_items = g_list_concat (store->completion_items, item);
+
+	userlist_set_user_button (userlist, sess);
 }
 
 static GtkTreeIter*
@@ -187,8 +194,9 @@ find_user (Store *store, struct User *user)
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		do {
 			gtk_tree_model_get (model, &iter, 2, &row_user, -1);
-			if (row_user == user)
+			if (row_user == user) {
 				return &iter;
+			}
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 	return NULL;
@@ -204,8 +212,9 @@ userlist_remove_user (Userlist *userlist, session *sess, struct User *user)
 	g_return_val_if_fail (store != NULL, FALSE);
 
 	iter = find_user (store, user);
-	if (!iter)
+	if (!iter) {
 		return FALSE;
+	}
 
 	gtk_list_store_remove (store->liststore, iter);
 
@@ -214,6 +223,8 @@ userlist_remove_user (Userlist *userlist, session *sess, struct User *user)
 	g_completion_remove_items (store->completion, item);
 	g_list_free (item);
 
+	userlist_set_user_button (userlist, sess);
+
 	return TRUE;
 }
 
@@ -267,8 +278,9 @@ userlist_clear_all (Userlist *userlist, session *sess)
 {
 	Store *store = g_hash_table_lookup (userlist->stores, sess);
 
-	if (store == NULL)
+	if (store == NULL) {
 		store = create_userlist (userlist, sess);
+	}
 
 	g_completion_clear_items (store->completion);
 	gtk_list_store_clear (store->liststore);
@@ -291,8 +303,9 @@ GtkListStore*
 userlist_get_store (Userlist *userlist, session *sess)
 {
 	Store *store = g_hash_table_lookup (userlist->stores, sess);
-	if (store == NULL)
+	if (store == NULL) {
 		store = create_userlist (userlist, sess);
+	}
 	return store->liststore;
 }
 
@@ -303,3 +316,19 @@ userlist_get_completion (Userlist *userlist, session *sess)
 	g_assert (store != NULL);
 	return store->completion;
 }
+
+void
+userlist_set_user_button (Userlist *userlist, session *sess)
+{
+	gchar *label;
+
+	if (gui.current_session == sess) {
+		if (sess->type == SESS_CHANNEL) {
+			label = g_strdup_printf (ngettext ("%d User", "%d Users", sess->total), sess->total);
+			gtk_button_set_label (GTK_BUTTON (gui.userlist_toggle), label);
+			g_free (label);
+		} else {
+			gtk_button_set_label (GTK_BUTTON (gui.userlist_toggle), _("Users"));
+		}
+	}
+}
